


library(emma)
library(MASS)




getGVar <- function(k,evar,hg,n){
  s = diag(n)-matrix(1/n,n,n);
  varg = hg^2*evar*(n-1)/((1-hg^2)*sum(diag(s%*%k%*%s)));
  return(varg);
}



getMinPvalues <- function(snps, nNullSimulation=50, kin) {

    nSample = dim(kin)[1]
    n = dim(snps)[2];
    null <- genNullPhenos(nSample, nNullSimulation,kin)
    ps <- emma.REML.t(null, snps, kin)
    ps <- ps$ps
    minp <- apply(ps, 2, min)
    return(minp)
}



getThreshold <- function(minPvalues) {

    sorted <- sort(minPvalues)
    return(sorted[ceiling(length(minPvalues)*0.05)]);

}






genNullPhenos <- function(nSample, nNullSimulation,kin) {

  evar = 1
  hs = 0.99999

  gvar = getGVar(kin,evar,hs,nSample);
  kca = chol(gvar*kin+evar*diag(1,nSample));
  y = mvrnorm(nNullSimulation,rep(0,nSample),kca)

  return(y);

}





powerSimulation <- function(nCausalSNPs, snps, threshold, kin, hs, vs) {

    evar = 1
    
    powerTab <- matrix(0, length(hs), length(vs))
    ind = sample(dim(snps)[1])[1:nCausalSNPs]

    for(i in ind){
        alt.y <- genAltPhenos(snps[i, ], evar, kin, hs, vs);
        ps <- emma.REML.t(alt.y, snps[i, ], kin)
        ps <- ps$ps
        count = 1;
        for(a in 1:length(hs)){
            for(b in 1:length(vs)){
                if (ps[count] < threshold) {
                    powerTab[a, b] <- powerTab[a,b] + 1
                }
                count = count + 1
            }
        }
    }

    powerTab = powerTab/length(ind)
    return(powerTab);
}





genAltPhenos <- function(snp, evar, k, hs, vs){

     y = genSNPPhenos(matrix(snp,nrow=1),evar, k, hs, vs);
    return(y);
}





genSNPPhenos <- function(snp,evar,k, hs,vs){
  n = dim(snp)[2];
  m = c();
  mult = 1;

  for(i in hs){
     gvar = getGVar(k,evar,i,n);
     kca = chol(gvar*k+evar*diag(1,n));
     for(j in vs){
        x = genPhenoMatrixSNPs(snp,kca,evar,j,mult);
        if(length(m) == 0){m = x;}
        else{m = rbind(m,x);}
     }
   }
  return(m);
}



genPhenoMatrixSNPs <- function(snp,kc,evar,varExplained,mult=1){
  n = dim(snp)[2];
  m = 1;
  y = matrix(rnorm(n),1,n);
  yp = t(t(kc) %*% t(y));
  betas = getBetas(yp,snp,evar,varExplained);
  ypp = yp + snp*betas;
  if(mult > 1){
    f = n / mult;
    ma = matrix(ypp[1,],f,mult);
    ypp = matrix(rowMeans(ma),1,f);
  }
  return(ypp);
}



getBetas <- function(y,snps,evar,varExplained){
  n = dim(y)[1];
  betas = lapply(c(1:n),function(x){varExplained*var(y[x,])/var(snps[x,])*(1-varExplained)});
  return(sqrt(unlist(betas)));
}


 




